// ===========================================================================
// CONTENT  : CLASS XMLObjectExporter
// AUTHOR   : Manfred Duchrow
// VERSION  : 2.0.1 - 14/05/2004
// HISTORY  :
//  07/06/2001  duma  CREATED
//	26/02/2003	duma	changed	->	removed getName(), added getVersion() and getVendor()
//	15/03/2003	duma	changed	->	removed dependencies on xerces
//	14/05/2004	duma	checked	->	adapted to JOI 2.1
//
// Copyright (c) 2001-2004, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pf.joi.xml;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.awt.FileDialog;
import java.awt.Frame;
import java.io.File;
import java.io.FileWriter;
import java.io.Writer;
import java.lang.reflect.Modifier;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.pf.joi.AbstractObjectSpy;
import org.pf.joi.ElementSpy;
import org.pf.joi.ExportProvider;
import org.pf.joi.Spy;
import org.pf.six.ModelConverter;
import org.pf.six.XmlStreamWriter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

/**
 * This class provides export functionality for inspected objects
 * to XML files.   <br>
 * For it's XML writing it requires any XML parser package  
 * that supports the JAXP factory mechanism and implements a SAX parser
 *
 * @author Manfred Duchrow
 * @version 2.0.1
 */
public class XMLObjectExporter extends XMLTags implements ExportProvider
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================

  // =========================================================================
  // CLASS METHODS
  // =========================================================================

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
  public XMLObjectExporter()
  {
    super() ;
  } // XMLObjectExporter()

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================

	/**
	 * Displays a file dialog to select the directory and filename and than
	 * writes the object contents to that file as XML data.
	 * 
	 * @param objectSpy The wrapped object to be exported
	 * @param parent The parent window. Usually that is the inspector window
	 */
  public boolean export( AbstractObjectSpy objectSpy, Frame parent )
    throws Exception
  {
    File file         = null ;

    file = this.selectFile( parent ) ;
    if ( file == null )
      return false ;

    return this.export( objectSpy, file ) ;
  } // export()

  // -------------------------------------------------------------------------

	/**
	 * Exports the given object to the file with the specified name.
	 * 
	 * @param objectSpy The wrapped object to be exported
	 * @param filename The name of the output file
	 */
  public boolean export( AbstractObjectSpy objectSpy, String filename )
    throws Exception
  {
    return this.export( objectSpy, new File( filename ) ) ;
  } // export()

  // -------------------------------------------------------------------------

	/**
	 * Exports the given object to the specified writer.
	 * 
	 * @param objectSpy The wrapped object to be exported
	 * @param writer The writer to which the object contents will be written
	 */
  public boolean export( AbstractObjectSpy objectSpy, Writer writer )
    throws Exception
  {
    Document doc      = null ;

    doc = this.createDocument( objectSpy ) ;
    this.writeDocToStream( doc, writer );
    return true ;
  } // export()

  // -------------------------------------------------------------------------

	/**
	 * Exports the given object to the specified file.
	 * 
	 * @param objectSpy The wrapped object to be exported
	 * @param file The file to which the object contents will be written
	 */
  public boolean export( AbstractObjectSpy objectSpy, File file )
    throws Exception
  {
    Writer writer     = null ;

    if ( file != null )
    {
      try
      {
        writer = new FileWriter( file ) ;
        this.export( objectSpy, writer );
        return true ;
      }
      catch ( Exception ex )
      {
        ex.printStackTrace();
        throw ex ;
      }
      finally
      {
        if ( writer != null )
          writer.close();
      }
    }

    return false ;
  } // export()

  // -------------------------------------------------------------------------

	/**
	 * Returns the label that will be shown in an inspector's 'File' menu.
	 */
  public String exportLabel()
  {
    return "Save as XML" ;
  } // exportLabel()

  // -------------------------------------------------------------------------

	/**
	 * Returns a simple XML tree representing the object's internal state that
	 * is encapsulated in the given object spy.
	 * 
	 * @param objectSpy The wrapper around the object to represent as XML
	 */
	public org.pf.six.Element createXMLTree( AbstractObjectSpy objectSpy )
    throws Exception
	{
    Document doc ;
    ModelConverter converter ;
		
		doc = this.createDocument( objectSpy ) ;
		converter = new ModelConverter() ;
		return converter.toSIXElement( doc ) ;
	} // createXMLTree()

	// -------------------------------------------------------------------------

	/**
	 * Returns a DOM document containing all data of the given object.
	 */
  public Document createDocument( AbstractObjectSpy objectSpy )
    throws Exception
  {
    Document doc                = null ;
    DocumentBuilder docBuilder  = null ;
    Element element             = null ;

    docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder() ;
    doc = docBuilder.newDocument() ;
    element = this.createMainElement( doc ) ;
    doc.appendChild( element ) ;
    this.appendInspectedObject( doc, element, objectSpy ) ;
    return doc ;
  } // createDocument()

  // -------------------------------------------------------------------------

	/**
	 * Returns the version of this export provider
	 */
  public String getVersion()
  {
    return "2.0.1" ;
  } // getVersion()

	// -------------------------------------------------------------------------
	
	/**
	 * Returns the vendor of this export provider
	 */
	public String getVendor()
	{
		return "Programmer's Friend" ;
	} // getVendor()

	// -------------------------------------------------------------------------

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================

  protected void appendInspectedObject( Document doc, Element rootElement,
                                        AbstractObjectSpy objectSpy )
    throws Exception
  {
    Element newElement   = null ;

    newElement = doc.createElement( tag_InspectedObject() ) ;
    rootElement.appendChild( newElement ) ;
    newElement.setAttribute( attr_Name(), objectSpy.getName() ) ;
    this.appendType( doc, newElement, objectSpy.getValueTypeString(),
                      objectSpy.getModifiers() ) ;
    this.appendObject( doc, newElement, objectSpy ) ;
  } // appendInspectedObject()

  // -------------------------------------------------------------------------

  protected void appendObject( Document doc, Element parentElement,
                                AbstractObjectSpy spy )
    throws Exception
  {
    Element objElement  = null ;
    Iterator iterator     = null ;
    ElementSpy elementSpy = null ;

    objElement = this.creaeObjectElement( doc, spy ) ;
    parentElement.appendChild( objElement ) ;

    iterator = spy. getElements().iterator() ;
    while ( iterator.hasNext() )
    {
      elementSpy = (ElementSpy)iterator.next() ;
      this.appendElement( doc, objElement, elementSpy ) ;
    }
  } // appendObject()

  // -------------------------------------------------------------------------

  protected Element creaeObjectElement( Document doc, Spy spy )
    throws Exception
  {
    Element objElement  = null ;

    objElement = doc.createElement( tag_Object() ) ;
    this.appendType( doc, objElement, spy.getValueTypeString(), 0 ) ;
    if ( spy.valueIsNull() )
      this.appendNull( doc, objElement ) ;
    else
      this.appendValue( doc, objElement, spy.getValueString()) ;

    return objElement ;
  } // createObjectElement()

  // -------------------------------------------------------------------------

  protected void appendElement( Document doc, Element parentElement,
                                ElementSpy elementSpy  )
    throws Exception
  {
    Element element     = null ;
    Element objElement  = null ;

    element = doc.createElement( tag_Element() ) ;
    parentElement.appendChild( element ) ;
    element.setAttribute( attr_Name(), this.trimQuotes( elementSpy.getName()) ) ;
    this.appendType( doc, element, elementSpy.getTypeString(), elementSpy.getModifiers() ) ;
    objElement = this.creaeObjectElement( doc, elementSpy ) ;
    element.appendChild( objElement ) ;
  } // appendElement()

  // -------------------------------------------------------------------------

  protected void appendType( Document doc, Element parentElement, String type,
                              int modifiers )
    throws Exception
  {
    Element newElement   = null ;

    newElement = doc.createElement( tag_Type() ) ;
    parentElement.appendChild( newElement ) ;
    newElement.setAttribute( attr_Name(), type ) ;
    this.appendModifiers( doc, newElement, modifiers ) ;
  } // appendType()

  // -------------------------------------------------------------------------

  protected void appendModifiers( Document doc, Element parentElement, int modifiers )
    throws Exception
  {
    Element newElement        = null ;
    String modifierName       = null ;
    StringTokenizer tokenizer = null ;

    tokenizer = new StringTokenizer( Modifier.toString( modifiers ) ) ;
    while ( tokenizer.hasMoreTokens() )
    {
      modifierName = tokenizer.nextToken() ;
      newElement = this.createModifierElement( doc, modifierName ) ;
      parentElement.appendChild( newElement ) ;
    }
  } // appendModifiers()

  // -------------------------------------------------------------------------

  protected Element createModifierElement( Document doc, String modifier )
    throws Exception
  {
    Element newElement   = null ;

    newElement = doc.createElement( tag_Modifier() ) ;
    newElement.setAttribute( attr_Name(), modifier ) ;
    return newElement ;
  } // createModifierElement()

  // -------------------------------------------------------------------------

  protected void appendValue( Document doc, Element parentElement,
                              String valueString )
    throws Exception
  {
    Element newElement   = null ;
    Text pcdata         = null ;

    newElement = doc.createElement( tag_DisplayValue() ) ;
    parentElement.appendChild( newElement ) ;

    pcdata = doc.createTextNode( this.trimQuotes( valueString ) ) ;
    newElement.appendChild( pcdata ) ;

  } // appendValue()

  // -------------------------------------------------------------------------

  protected void appendNull( Document doc, Element parentElement )
    throws Exception
  {
    Element newElement   = null ;

    newElement = doc.createElement( this.tag_Null() ) ;
    parentElement.appendChild( newElement ) ;
  } // appendNull()

  // -------------------------------------------------------------------------

  protected Element createMainElement( Document doc )
    throws Exception
  {
    return doc.createElement( this.tag_DocType() ) ;
  } // createMainElement()

  // -------------------------------------------------------------------------

  protected void writeDocToStream( Document doc, Writer writer )
  {
    XmlStreamWriter.writeDocumentStart( writer, "ISO-8859-1" ) ;
    XmlStreamWriter.writeDocumentType( writer, this.tag_DocType(), "joi.dtd" ) ;
		XmlStreamWriter.writeDOM( doc, writer ) ;
  } // writeDocToStream()

  // -------------------------------------------------------------------------

  protected File selectFile( Frame parent )
  {
    FileDialog fileDialog   = null ;
    File file               = null ;
    String fileName         = null ;

    fileDialog = new FileDialog( parent, "Save as XML", FileDialog.SAVE ) ;
    fileDialog.show();
    fileName = fileDialog.getFile() ;
    if ( fileName != null )
      file = new File( fileDialog.getDirectory(), fileName ) ;

    return file ;
  } // selectFile()

  // -------------------------------------------------------------------------

  protected String trimQuotes( String text )
  {
    if ( ( text.startsWith( "\"" ) ) && ( text.endsWith( "\"" ) ) )
      return text.substring( 1, text.length()-1 ) ;
    else
      return text ;
  } // trimQuotes()

  // -------------------------------------------------------------------------

} // class XMLObjectExporter